@@ -1,26 +1,31 @@
 # gemato: CLI routines
 # vim:fileencoding=utf-8
-# (c) 2017-2018 Michał Górny
+# (c) 2017-2020 Michał Górny
 # Licensed under the terms of 2-clause BSD license
 
 from __future__ import print_function
 
 import argparse
 import datetime
-import io
 import logging
 import multiprocessing
 import os.path
 import sys
 import timeit
 
-import gemato.exceptions
-import gemato.find_top_level
-import gemato.hash
-import gemato.manifest
-import gemato.openpgp
-import gemato.profile
-import gemato.recursiveloader
+from gemato.exceptions import GematoException
+from gemato.find_top_level import find_top_level_manifest
+from gemato.hash import hash_file, hash_path
+from gemato.manifest import (
+    ManifestFileEntry,
+    manifest_hashes_to_hashlib,
+    )
+from gemato.openpgp import (
+    OpenPGPEnvironment,
+    OpenPGPSystemEnvironment,
+    )
+from gemato.profile import get_profile_by_name
+from gemato.recursiveloader import ManifestRecursiveLoader
 
 
 def verify_failure(e):
@@ -28,7 +33,7 @@ def verify_failure(e):
     return False
 
 
-class GematoCommand(object):
+class GematoCommand:
     """
     Base class for commands supported by gemato.
     """
@@ -51,14 +56,16 @@ class GematoCommand(object):
         """
         Add options specific to the command to subparser @subp.
         """
-        pass
+        subp.add_argument('--debug', action='store_true',
+                          help='Enable debugging output')
 
     def parse_args(self, args, argp):
         """
         Process command-line arguments @args. @argp is the argparse
         instance provided for error reporting.
         """
-        pass
+        if args.debug:
+            logging.getLogger().setLevel(logging.DEBUG)
 
     def __call__(self):
         """
@@ -73,38 +80,43 @@ class GematoCommand(object):
         pass
 
 
-class BaseOpenPGPMixin(object):
+class BaseOpenPGPMixin:
     """
     A base mixin that adds logic to load and use OpenPGP keys.
     """
 
     def __init__(self):
-        super(BaseOpenPGPMixin, self).__init__()
+        super().__init__()
         self.openpgp_env = None
 
     def add_options(self, subp):
-        super(BaseOpenPGPMixin, self).add_options(subp)
+        super().add_options(subp)
 
-        subp.add_argument('-K', '--openpgp-key',
-                help='Use only the OpenPGP key(s) from a specific file')
+        subp.add_argument(
+            '-K', '--openpgp-key',
+            help='Use only the OpenPGP key(s) from a specific file')
+        subp.add_argument(
+            '--proxy',
+            help='Use HTTP proxy')
 
     def parse_args(self, args, argp):
-        super(BaseOpenPGPMixin, self).parse_args(args, argp)
+        super().parse_args(args, argp)
 
         # use isolated environment if key is specified;
         # system environment otherwise
         if args.openpgp_key is not None:
-            env_class = gemato.openpgp.OpenPGPEnvironment
+            env_class = OpenPGPEnvironment
         else:
-            env_class = gemato.openpgp.OpenPGPSystemEnvironment
-        self.openpgp_env = env_class()
+            env_class = OpenPGPSystemEnvironment
+        self.openpgp_env = env_class(debug=args.debug,
+                                     proxy=args.proxy)
 
         if args.openpgp_key is not None:
-            with io.open(args.openpgp_key, 'rb') as f:
+            with open(args.openpgp_key, 'rb') as f:
                 self.openpgp_env.import_key(f)
 
     def cleanup(self):
-        super(BaseOpenPGPMixin, self).cleanup()
+        super().cleanup()
 
         if self.openpgp_env is not None:
             self.openpgp_env.close()
@@ -116,41 +128,54 @@ class VerifyingOpenPGPMixin(BaseOpenPGPMixin):
     """
 
     def add_options(self, subp):
-        super(VerifyingOpenPGPMixin, self).add_options(subp)
+        super().add_options(subp)
 
-        subp.add_argument('-R', '--no-refresh-keys', action='store_false',
-                dest='refresh_keys',
-                help='Disable refreshing OpenPGP key (prevents network access, '
-                    +'applicable when using -K only)')
+        subp.add_argument(
+            '-R', '--no-refresh-keys', action='store_false',
+            dest='refresh_keys',
+            help='Disable refreshing OpenPGP key (prevents network '
+                 'access, applicable when using -K only)')
+        subp.add_argument(
+            '-W', '--no-wkd', action='store_false',
+            dest='allow_wkd',
+            help='Do not attempt to use WKD to refetch keys (use '
+                 'keyservers only)')
+        subp.add_argument(
+            '--keyserver',
+            help='Force custom keyserver URL')
 
     def parse_args(self, args, argp):
-        super(VerifyingOpenPGPMixin, self).parse_args(args, argp)
+        super().parse_args(args, argp)
 
         if args.openpgp_key is not None:
             # always refresh keys to check for revocation
             # (unless user specifically asked us not to)
             if args.refresh_keys:
-                logging.info('Refreshing keys from keyserver...')
-                self.openpgp_env.refresh_keys()
+                logging.info('Refreshing keys...')
+                self.openpgp_env.refresh_keys(allow_wkd=args.allow_wkd,
+                                              keyserver=args.keyserver)
                 logging.info('Keys refreshed.')
 
 
-class BaseManifestLoaderMixin(object):
+class BaseManifestLoaderMixin:
     """
     Mixin for commands using RecursiveManifestLoader class.
     """
 
     def add_options(self, subp):
-        super(BaseManifestLoaderMixin, self).add_options(subp)
+        super().add_options(subp)
 
-        subp.add_argument('-j', '--jobs', type=int,
-                help='Specify the maximum number of parallel jobs to use (default: {})'
-                    .format(multiprocessing.cpu_count()))
-        subp.add_argument('-x', '--one-file-system', action='store_true',
-                help='Do not cross filesystem boundaries (report an error instead)')
+        subp.add_argument(
+            '-j', '--jobs', type=int,
+            help=f'Specify the maximum number of parallel jobs to use '
+                 f'(default: {multiprocessing.cpu_count()})')
+        subp.add_argument(
+            '-x', '--one-file-system', action='store_true',
+            help='Do not cross filesystem boundaries (report an error '
+                 'instead)')
 
     def parse_args(self, args, argp):
-        super(BaseManifestLoaderMixin, self).parse_args(args, argp)
+        super().parse_args(args, argp)
 
         self.init_kwargs = {}
         if args.jobs is not None:
@@ -167,20 +192,25 @@ class VerifyCommand(BaseManifestLoaderMixin, VerifyingOpenPGPMixin,
     help = 'Verify one or more directories against Manifests'
 
     def add_options(self, verify):
-        super(VerifyCommand, self).add_options(verify)
+        super().add_options(verify)
 
-        verify.add_argument('paths', nargs='*', default=['.'],
-                help='Paths to verify (defaults to "." if none specified)')
-        verify.add_argument('-k', '--keep-going', action='store_true',
-                help='Continue reporting errors rather than terminating on the first failure')
-        verify.add_argument('-P', '--no-openpgp-verify', action='store_false',
-                dest='openpgp_verify',
-                help='Disable OpenPGP verification of signed Manifests')
-        verify.add_argument('-s', '--require-signed-manifest', action='store_true',
-                help='Require that the top-level Manifest is OpenPGP signed')
+        verify.add_argument(
+            'paths', nargs='*', default=['.'],
+            help='Paths to verify (defaults to "." if none specified)')
+        verify.add_argument(
+            '-k', '--keep-going', action='store_true',
+            help='Continue reporting errors rather than terminating '
+                 'on the first failure')
+        verify.add_argument(
+            '-P', '--no-openpgp-verify', action='store_false',
+            dest='openpgp_verify',
+            help='Disable OpenPGP verification of signed Manifests')
+        verify.add_argument(
+            '-s', '--require-signed-manifest', action='store_true',
+            help='Require that the top-level Manifest is OpenPGP signed')
 
     def parse_args(self, args, argp):
-        super(VerifyCommand, self).parse_args(args, argp)
+        super().parse_args(args, argp)
 
         self.paths = args.paths
         self.require_signed_manifest = args.require_signed_manifest
@@ -193,37 +223,40 @@ class VerifyCommand(BaseManifestLoaderMixin, VerifyingOpenPGPMixin,
             self.init_kwargs['verify_openpgp'] = False
 
     def __call__(self):
-        super(VerifyCommand, self).__call__()
+        super().__call__()
 
         ret = True
 
         for p in self.paths:
-            tlm = gemato.find_top_level.find_top_level_manifest(p)
+            tlm = find_top_level_manifest(p)
             if tlm is None:
-                logging.error('Top-level Manifest not found in {}'.format(p))
+                logging.error(f'Top-level Manifest not found in {p}')
                 return 1
 
             start = timeit.default_timer()
-            m = gemato.recursiveloader.ManifestRecursiveLoader(tlm,
-                    **self.init_kwargs)
+            m = ManifestRecursiveLoader(tlm, **self.init_kwargs)
             if self.require_signed_manifest and not m.openpgp_signed:
-                logging.error('Top-level Manifest {} is not OpenPGP signed'.format(tlm))
+                logging.error(f'Top-level Manifest {tlm} is not '
+                              f'OpenPGP signed')
                 return 1
 
             ts = m.find_timestamp()
             if ts:
-                logging.info('Manifest timestamp: {} UTC'.format(ts.ts))
+                logging.info(f'Manifest timestamp: {ts.ts} UTC')
 
             if m.openpgp_signed:
                 logging.info('Valid OpenPGP signature found:')
-                logging.info('- primary key: {}'.format(
-                    m.openpgp_signature.primary_key_fingerprint))
-                logging.info('- subkey: {}'.format(
-                    m.openpgp_signature.fingerprint))
-                logging.info('- timestamp: {} UTC'.format(
-                    m.openpgp_signature.timestamp))
+                logging.info(
+                    f'- primary key: '
+                    f'{m.openpgp_signature.primary_key_fingerprint}')
+                logging.info(
+                    f'- subkey: '
+                    f'{m.openpgp_signature.fingerprint}')
+                logging.info(
+                    f'- timestamp: '
+                    f'{m.openpgp_signature.timestamp} UTC')
 
-            logging.info('Verifying {}...'.format(p))
+            logging.info(f'Verifying {p}...')
 
             relpath = os.path.relpath(p, os.path.dirname(tlm))
             if relpath == '.':
@@ -231,7 +264,7 @@ class VerifyCommand(BaseManifestLoaderMixin, VerifyingOpenPGPMixin,
             ret &= m.assert_directory_verifies(relpath, **self.kwargs)
 
             stop = timeit.default_timer()
-            logging.info('{} verified in {:.2f} seconds'.format(p, stop - start))
+            logging.info(f'{p} verified in {stop - start:.2f} seconds')
 
         return 0 if ret else 1
 
@@ -242,32 +275,43 @@ class BaseUpdateMixin(BaseManifestLoaderMixin, BaseOpenPGPMixin):
     """
 
     def add_options(self, update):
-        super(BaseUpdateMixin, self).add_options(update)
+        super().add_options(update)
 
-        update.add_argument('-c', '--compress-watermark', type=int,
-                help='Minimum Manifest size for files to be compressed')
-        update.add_argument('-C', '--compress-format',
-                help='Format for compressed files (e.g. "gz", "bz2"...)')
-        update.add_argument('-f', '--force-rewrite', action='store_true',
-                help='Force rewriting all the Manifests, even if they did not change')
-        update.add_argument('-H', '--hashes',
-                help='Whitespace-separated list of hashes to use')
-        update.add_argument('-k', '--openpgp-id',
-                help='Use the specified OpenPGP key (by ID or user)')
-        update.add_argument('-p', '--profile',
-                help='Use the specified profile ("default", "ebuild", "old-ebuild"...)')
+        update.add_argument(
+            '-c', '--compress-watermark', type=int,
+            help='Minimum Manifest size for files to be compressed')
+        update.add_argument(
+            '-C', '--compress-format',
+            help='Format for compressed files (e.g. "gz", "bz2"...)')
+        update.add_argument(
+            '-f', '--force-rewrite', action='store_true',
+            help='Force rewriting all the Manifests, even if they did '
+                 'not change')
+        update.add_argument(
+            '-H', '--hashes',
+            help='Whitespace-separated list of hashes to use')
+        update.add_argument(
+            '-k', '--openpgp-id',
+            help='Use the specified OpenPGP key (by ID or user)')
+        update.add_argument(
+            '-p', '--profile',
+            help='Use the specified profile ("default", "ebuild", '
+                 '"old-ebuild"...)')
         signgroup = update.add_mutually_exclusive_group()
-        signgroup.add_argument('-s', '--sign', action='store_true',
-                default=None,
-                help='Force signing the top-level Manifest')
-        signgroup.add_argument('-S', '--no-sign', action='store_false',
-                dest='sign',
-                help='Disable signing the top-level Manifest')
-        update.add_argument('-t', '--timestamp', action='store_true',
-                help='Include TIMESTAMP entry in Manifest')
+        signgroup.add_argument(
+            '-s', '--sign', action='store_true',
+            default=None,
+            help='Force signing the top-level Manifest')
+        signgroup.add_argument(
+            '-S', '--no-sign', action='store_false',
+            dest='sign',
+            help='Disable signing the top-level Manifest')
+        update.add_argument(
+            '-t', '--timestamp', action='store_true',
+            help='Include TIMESTAMP entry in Manifest')
 
     def parse_args(self, args, argp):
-        super(BaseUpdateMixin, self).parse_args(args, argp)
+        super().parse_args(args, argp)
 
         self.timestamp = args.timestamp
 
@@ -287,8 +331,8 @@ class BaseUpdateMixin(BaseManifestLoaderMixin, BaseOpenPGPMixin):
         if args.openpgp_id is not None:
             self.init_kwargs['openpgp_keyid'] = args.openpgp_id
         if args.profile is not None:
-            self.init_kwargs['profile'] = gemato.profile.get_profile_by_name(
-                    args.profile)
+            self.init_kwargs['profile'] = (
+                get_profile_by_name(args.profile))
         if args.sign is not None:
             self.init_kwargs['sign_openpgp'] = args.sign
 
@@ -298,56 +342,62 @@ class UpdateCommand(BaseUpdateMixin, GematoCommand):
     help = 'Update the Manifest entries for one or more directory trees'
 
     def add_options(self, update):
-        super(UpdateCommand, self).add_options(update)
+        super().add_options(update)
 
-        update.add_argument('paths', nargs='*', default=['.'],
-                help='Paths to update (defaults to "." if none specified)')
-        update.add_argument('-i', '--incremental', action='store_true',
-                help='Perform incremental update by comparing mtimes against TIMESTAMP')
+        update.add_argument(
+            'paths', nargs='*', default=['.'],
+            help='Paths to update (defaults to "." if none specified)')
+        update.add_argument(
+            '-i', '--incremental', action='store_true',
+            help='Perform incremental update by comparing mtimes '
+                 'against TIMESTAMP')
 
     def parse_args(self, args, argp):
-        super(UpdateCommand, self).parse_args(args, argp)
+        super().parse_args(args, argp)
 
         self.paths = args.paths
         self.incremental = args.incremental
 
     def __call__(self):
-        super(UpdateCommand, self).__call__()
+        super().__call__()
 
         for p in self.paths:
-            tlm = gemato.find_top_level.find_top_level_manifest(p)
+            tlm = find_top_level_manifest(p)
             if tlm is None:
-                logging.error('Top-level Manifest not found in {}'.format(p))
+                logging.error(f'Top-level Manifest not found in {p}')
                 return 1
 
             start = timeit.default_timer()
-            m = gemato.recursiveloader.ManifestRecursiveLoader(tlm,
-                    **self.init_kwargs)
+            m = ManifestRecursiveLoader(tlm, **self.init_kwargs)
 
             # if not specified by user, profile must set it
             if m.hashes is None:
-                logging.error('--hashes must be specified if not implied by --profile')
+                logging.error('--hashes must be specified if not '
+                              'implied by --profile')
                 return 1
 
             relpath = os.path.relpath(p, os.path.dirname(tlm))
             if relpath == '.':
                 relpath = ''
             if self.timestamp and relpath != '':
-                logging.error('Timestamp can only be updated if doing full-tree update')
+                logging.error('Timestamp can only be updated if doing '
+                              'full-tree update')
                 return 1
 
             update_kwargs = {}
             if self.incremental:
                 if relpath != '':
-                    logging.error('Incremental works only for full-tree update')
+                    logging.error('Incremental works only for '
+                                  'full-tree update')
                     return 1
                 last_ts = m.find_timestamp()
                 if last_ts is None:
-                    logging.error('Incremental specified but no timestamp in Manifest')
+                    logging.error('Incremental specified but no '
+                                  'timestamp in Manifest')
                     return 1
                 update_kwargs['last_mtime'] = last_ts.ts.timestamp()
 
-            logging.info('Updating Manifests in {}...'.format(p))
+            logging.info(f'Updating Manifests in {p}...')
 
             start_ts = datetime.datetime.utcnow()
             m.update_entries_for_directory(relpath, **update_kwargs)
@@ -366,7 +416,7 @@ class UpdateCommand(BaseUpdateMixin, GematoCommand):
             m.save_manifests(**self.save_kwargs)
 
             stop = timeit.default_timer()
-            logging.info('{} updated in {:.2f} seconds'.format(p, stop - start))
+            logging.info(f'{p} updated in {stop - start:.2f} seconds')
 
         return 0
 
@@ -376,31 +426,34 @@ class CreateCommand(BaseUpdateMixin, GematoCommand):
     help = 'Create a Manifest tree starting at the specified file'
 
     def add_options(self, create):
-        super(CreateCommand, self).add_options(create)
+        super().add_options(create)
 
-        create.add_argument('paths', nargs='*', default=['.'],
-                help='Paths to create Manifest in (defaults to "." if none specified)')
+        create.add_argument(
+            'paths', nargs='*', default=['.'],
+            help='Paths to create Manifest in (defaults to "." '
+                 'if none specified)')
 
     def parse_args(self, args, argp):
-        super(CreateCommand, self).parse_args(args, argp)
+        super().parse_args(args, argp)
 
         self.init_kwargs['allow_create'] = True
         self.paths = args.paths
 
     def __call__(self):
-        super(CreateCommand, self).__call__()
+        super().__call__()
 
         for p in self.paths:
             start = timeit.default_timer()
-            m = gemato.recursiveloader.ManifestRecursiveLoader(
-                    os.path.join(p, 'Manifest'), **self.init_kwargs)
+            m = ManifestRecursiveLoader(
+                os.path.join(p, 'Manifest'), **self.init_kwargs)
 
             # if not specified by user, profile must set it
             if m.hashes is None:
-                logging.error('--hashes must be specified if not implied by --profile')
+                logging.error('--hashes must be specified if not '
+                              'implied by --profile')
                 return 1
 
-            logging.info('Creating Manifests in {}...'.format(p))
+            logging.info(f'Creating Manifests in {p}...')
 
             start_ts = datetime.datetime.utcnow()
             m.update_entries_for_directory()
@@ -412,7 +465,7 @@ class CreateCommand(BaseUpdateMixin, GematoCommand):
             m.save_manifests(**self.save_kwargs)
 
             stop = timeit.default_timer()
-            logging.info('{} updated in {:.2f} seconds'.format(p, stop - start))
+            logging.info(f'{p} updated in {stop - start:.2f} seconds')
 
         return 0
 
@@ -422,87 +475,85 @@ class HashCommand(GematoCommand):
     help = 'Generate hashes for specified file(s) and/or stdin'
 
     def add_options(self, subp):
-        super(HashCommand, self).add_options(subp)
+        super().add_options(subp)
 
-        subp.add_argument('paths', nargs='*', default=['-'],
-                help='Paths to hash (defaults to "-" (stdin) if not specified)')
-        subp.add_argument('-H', '--hashes', required=True,
-                help='Whitespace-separated list of hashes to use')
+        subp.add_argument(
+            'paths', nargs='*', default=['-'],
+            help='Paths to hash (defaults to "-" (stdin) if not '
+                 'specified)')
+        subp.add_argument(
+            '-H', '--hashes', required=True,
+            help='Whitespace-separated list of hashes to use')
 
     def parse_args(self, args, argp):
-        super(HashCommand, self).parse_args(args, argp)
+        super().parse_args(args, argp)
 
         self.hashes = sorted(args.hashes.split())
         self.paths = args.paths
 
     def __call__(self):
-        super(HashCommand, self).__call__()
+        super().__call__()
 
-        hashlib_hashes = list(
-                gemato.manifest.manifest_hashes_to_hashlib(self.hashes))
+        hashlib_hashes = list(manifest_hashes_to_hashlib(self.hashes))
         hashlib_hashes.append('__size__')
 
         for p in self.paths:
             if p == '-':
-                if sys.hexversion >= 0x03000000:
-                    f = sys.stdin.buffer
-                else:
-                    f = sys.stdin
-                h = gemato.hash.hash_file(f, hashlib_hashes)
+                h = hash_file(sys.stdin.buffer, hashlib_hashes)
             else:
-                h = gemato.hash.hash_path(p, hashlib_hashes)
+                h = hash_path(p, hashlib_hashes)
 
             sz = h.pop('__size__')
-            e = gemato.manifest.ManifestFileEntry(p, sz,
-                    dict((mh, h[hh]) for mh, hh in zip(self.hashes, hashlib_hashes)))
+            e = ManifestFileEntry(
+                p, sz,
+                dict((mh, h[hh]) for mh, hh
+                     in zip(self.hashes, hashlib_hashes)))
             print(' '.join(e.to_list('DATA' if p != '-' else 'STDIN')))
 
 
 class OpenPGPVerifyCommand(VerifyingOpenPGPMixin, GematoCommand):
     name = 'openpgp-verify'
-    help = 'Verify OpenPGP signatures embedded in specified file(s) and/or stdin'
+    help = ('Verify OpenPGP signatures embedded in specified file(s) '
+            'and/or stdin')
 
     def add_options(self, subp):
-        super(OpenPGPVerifyCommand, self).add_options(subp)
+        super().add_options(subp)
 
-        subp.add_argument('paths', nargs='*', default=['-'],
-                help='Paths to hash (defaults to "-" (stdin) if not specified)')
+        subp.add_argument(
+            'paths', nargs='*', default=['-'],
+            help='Paths to hash (defaults to "-" (stdin) if not '
+                 'specified)')
 
     def parse_args(self, args, argp):
-        super(OpenPGPVerifyCommand, self).parse_args(args, argp)
+        super().parse_args(args, argp)
 
         self.paths = args.paths
 
     def __call__(self):
-        super(OpenPGPVerifyCommand, self).__call__()
+        super().__call__()
 
         ret = True
 
         for p in self.paths:
             if p == '-':
-                if sys.hexversion >= 0x03000000:
-                    f = sys.stdin
-                else:
-                    f = io.open(sys.stdin.fileno(), 'r')
+                f = sys.stdin
             else:
-                f = io.open(p, 'r')
+                f = open(p, 'r')
 
             try:
                 try:
                     sig = self.openpgp_env.verify_file(f)
-                except gemato.exceptions.GematoException as e:
-                    logging.error(u'OpenPGP verification failed for {}:\n{}'
-                            .format(p, e))
+                except GematoException as e:
+                    logging.error(
+                        f'OpenPGP verification failed for {p}:\n{e}')
                     ret = False
                 else:
-                    logging.info('Valid OpenPGP signature found in {}:'
-                            .format(p))
-                    logging.info('- primary key: {}'.format(
-                        sig.primary_key_fingerprint))
-                    logging.info('- subkey: {}'.format(
-                        sig.fingerprint))
-                    logging.info('- timestamp: {} UTC'.format(
-                        sig.timestamp))
+                    logging.info(
+                        f'Valid OpenPGP signature found in {p}:')
+                    logging.info(
+                        f'- primary key: {sig.primary_key_fingerprint}')
+                    logging.info(f'- subkey: {sig.fingerprint}')
+                    logging.info(f'- timestamp: {sig.timestamp} UTC')
             finally:
                 if p != '-':
                     f.close()
@@ -535,15 +586,11 @@ def main(argv):
             return vals.cmd()
         finally:
             vals.cmd.cleanup()
-    except gemato.exceptions.GematoException as e:
+    except GematoException as e:
         logging.error(e)
         return 1
 
 
 def setuptools_main():
     logging.getLogger().setLevel(logging.INFO)
-    if sys.hexversion < 0x03000000:
-        argv = [x.decode(sys.getfilesystemencoding()) for x in sys.argv]
-    else:
-        argv = sys.argv
-    sys.exit(main(argv))
+    sys.exit(main(sys.argv))
